#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from fastapi import APIRouter, Request, Depends, Query, HTTPException, status
from common.security import valid_access_token
from fastapi.responses import JSONResponse
from sqlalchemy.orm import Session
from sqlalchemy import and_, or_
from sqlalchemy.sql import func
from common.auth_user import *
from pydantic import BaseModel
from database import get_db
from typing import List
from models import *
from utils import *
from utils.ry_system_util import *
from utils.three_proofing_responsible_util import *
import traceback
import json

router = APIRouter()

@router.get('/list')
async def get_dict_data_by_type(
    type_parent_id: str = Query(None, max_length=100),
    pageNum: int = Query(1, gt=0),
    pageSize: int = Query(10, gt=0),
    db: Session = Depends(get_db),
    body = Depends(remove_xss_json),
    user_id = Depends(valid_access_token)
):
    try:
        # 根据 dict_type 查询字典数据
        # dict_data = db.query(SysDictData).filter_by(dict_type=dictType).all()
        query = db.query(ThreeProofingResponsibleType)

        query = query.filter(ThreeProofingResponsibleType.del_flag != '2')
        # 添加查询条件
        if type_parent_id:
            query = query.filter(ThreeProofingResponsibleType.type_parent_id==type_parent_id)

        # 计算总记录数
        total_count = query.count()

        # 计算分页
        offset = (pageNum - 1) * pageSize
        dict_data = query.offset(offset).limit(pageSize).all()

        # 转换为字典
        dict_data_list = []
        for d in dict_data:
            dict_data = get_dict_data_info(db, 'three_proofing', d.type_parent_id)
            dict_data_list.append({
                "id": d.id,
                "type_parent_id": d.type_parent_id,
                "type_parent":dict_data.dict_label,
                "type_name": d.type_name,
                "order_num": d.order_num,
                "createTime": d.create_time.strftime('%Y-%m-%d %H:%M:%S') if d.create_time else ''
            })
        # 构建返回结果
        result = {
            "total": total_count,
            "rows": dict_data_list,
            "code": 200,
            "msg": "查询成功"
        }
        return result

    except Exception as e:
        # 处理异常
        raise HTTPException(status_code=status.HTTP_500_INTERNAL_SERVER_ERROR, detail=str(e))

@router.get('/info/{id}')
async def get_dict_data_by_type(
    id: str ,
    db: Session = Depends(get_db),
    body = Depends(remove_xss_json),
    user_id = Depends(valid_access_token)
):
    try:
        # 根据 dict_type 查询字典数据
        # dict_data = dict_type_get_dict_data_info(db,'three_proofing')
        query = db.query(ThreeProofingResponsibleType)

        # 添加查询条件
        # if dictType:
        query = query.filter(ThreeProofingResponsibleType.id==id)
        query = query.filter(ThreeProofingResponsibleType.del_flag != '2')


        d = query.first()
        dict_data = get_dict_data_info(db, 'three_proofing', d.type_parent_id)
        # 转换为字典
        dict_data_list = {
                "id": d.id,
                "type_parent_id": d.type_parent_id,
                "type_parent":dict_data.dict_label,
                "type_name": d.type_name,
                "order_num": d.order_num,
                "createTime": d.create_time.strftime('%Y-%m-%d %H:%M:%S') if d.create_time else ''
            }

        # 构建返回结果
        result = {
            "data": dict_data_list,
            "code": 200,
            "msg": "查询成功"
        }
        return result

    except Exception as e:
        # 处理异常
        raise HTTPException(status_code=status.HTTP_500_INTERNAL_SERVER_ERROR, detail=str(e))


@router.post('/create')
async def create_dict_data(
    db: Session = Depends(get_db),
    body = Depends(remove_xss_json),
    user_id = Depends(valid_access_token)
):
    try:
        # 创建一个新的 SysDictData 实例
        new_dict_data = ThreeProofingResponsibleType(
            type_parent_id=body['type_parent_id'],
            type_name=body['type_name'],
            order_num=body['order_num'],
            create_by=user_id
        )

        # 添加到会话并提交
        db.add(new_dict_data)
        db.commit()
        db.refresh(new_dict_data)  # 可选，如果需要刷新实例状态

        # 构建返回结果
        result = {
            "code": 200,
            "msg": "操作成功",
            "data": None
        }
        return result
    except Exception as e:
        # 处理异常
        raise HTTPException(status_code=status.HTTP_500_INTERNAL_SERVER_ERROR, detail=str(e))

@router.put("/update")
async def updata_dict_type(
    db: Session = Depends(get_db),
    body = Depends(remove_xss_json),
    user_id = Depends(valid_access_token)
):
    try:
        # 从请求数据创建一个新的 SysDictType 实例
        query = db.query(ThreeProofingResponsibleType)
        query = query.filter(ThreeProofingResponsibleType.id == body['id'])
        query = query.filter(ThreeProofingResponsibleType.del_flag != '2')

        # query = db.query(SysDictData).filter(SysDictData.dict_code == form_data.dictCode)
        # query = db.query(SysDictData).filter(SysDictData.del_flag != '2')

        dict_data = query.first()
        if not dict_data:
            return JSONResponse(status_code=404, content={
                'errcode': 404,
                'errmsg': 'type不存在'
            })
        dict_data.type_parent_id=body['type_parent_id']
        dict_data.type_name = body['type_name']
        dict_data.order_num = body['order_num']
        dict_data.update_by = user_id
        # 添加到数据库会话并提交
        db.commit()
        # db.refresh(new_dict_type)  # 可选，如果需要刷新实例状态

        # 构建并返回响应
        return {
            "code": 200,
            "msg": "操作成功",
            "data": None  # 根据你的响应示例，data 为 null
        }
    except Exception as e:
        # 处理异常
        raise HTTPException(status_code=500, detail=str(e))


@router.delete("/delete/{id}")  # 使用 ID 来标识要删除的接口
async def delete_dict_data(
    id: str,
    db: Session = Depends(get_db),
    body = Depends(remove_xss_json),
    user_id = Depends(valid_access_token)
):
    try:
        # 从数据库中获取要删除的 OneShareApiEntity 实例
        query = db.query(ThreeProofingResponsibleType)
        query = query.filter(ThreeProofingResponsibleType.id == id)
        query = query.filter(ThreeProofingResponsibleType.del_flag != '2')
        dict_data = query.first()
        # dict_data = db.query(SysDictData).filter(SysDictData.dict_code == dictCode and SysDictData.del_flag != '2').first()

        if not dict_data:
            return JSONResponse(status_code=404, content={
                'errcode': 404,
                'errmsg': 'type不存在'
            })
        dict_data.del_flag = '2'
        # 删除实例
        # db.delete(api)
        db.commit()

        # 构建并返回响应
        return {
            "code": 200,
            "msg": "操作成功",
            "data": None
        }
    except Exception as e:

        raise HTTPException(status_code=status.HTTP_500_INTERNAL_SERVER_ERROR, detail=str(e))

@router.delete("/data/{dictCode}")  # 使用 ID 来标识要删除的接口
async def delete_dict_data(
    dictCode: str,
    db: Session = Depends(get_db),
    body = Depends(remove_xss_json),
    user_id = Depends(valid_access_token)
):
    try:
        # 从数据库中获取要删除的 OneShareApiEntity 实例
        query = db.query(ThreeProofingResponsibleType)
        query = query.filter(ThreeProofingResponsibleType.del_flag != '2')
        dictCodeList = dictCode.split(',')
        query = query.filter(ThreeProofingResponsibleType.id.in_(dictCodeList))
        dict_data = query.all()
        # dict_data = db.query(SysDictData).filter(SysDictData.dict_code == dictCode and SysDictData.del_flag != '2').first()

        if not dict_data:
            return JSONResponse(content={"msg":"type不存在"},status_code=404)
            # detail = "dict不存在"
            # raise HTTPException(status_code=404, detail="dict不存在")
        for dict in dict_data:
            dict.del_flag = '2'
        # 删除实例
        # db.delete(api)
        db.commit()

        # 构建并返回响应
        return {
            "code": 200,
            "msg": "操作成功",
            "data": None
        }
    except Exception as e:
        # 处理异常
        traceback.print_exc()
        raise HTTPException(status_code=status.HTTP_500_INTERNAL_SERVER_ERROR, detail=str(e))

@router.get('/treeData')
async def get_dict_data_by_type(
    db: Session = Depends(get_db),
    body = Depends(remove_xss_json),
    user_id = Depends(valid_access_token)
):
    try:
        # 根据 dict_type 查询字典数据
        # dict_data = db.query(SysDictData).filter_by(dict_type=dictType).all()

        dict_data_list = []
        dict_data = dict_type_get_dict_data_info(db, 'three_proofing')
        for parent_info in dict_data:
            data = {"id":parent_info.dict_value,"label":parent_info.dict_label,"css_class":parent_info.css_class}
            type_list = get_type_parent_id_by_type_list(db,parent_info.dict_value)
            if type_list:
                children = []
                for type_info in type_list:
                    children.append({"id":str(type_info.id),"label":type_info.type_name,"parent_id":type_info.type_parent_id})
                data['children']=children
            other_type_list = get_other_type_list_by_type_parent_id(db,parent_info.dict_value)
            if other_type_list:
                children = []
                for type_info in other_type_list:
                    children.append({"id":str(type_info.id),"label":type_info.type_name,"parent_id":type_info.type_parent_id})
                data['children2']=children
            dict_data_list.append(data)
        result = {
            "rows": dict_data_list,
            "code": 200,
            "msg": "查询成功"
        }
        return result

    except Exception as e:
        # 处理异常
        traceback.print_exc()
        raise HTTPException(status_code=status.HTTP_500_INTERNAL_SERVER_ERROR, detail=str(e))
